authorization_urls = dict(
    homologation='https://wsaahomo.afip.gov.ar/ws/services/LoginCms?wsdl',
    production='https://wsaa.afip.gov.ar/ws/services/LoginCms?wsdl',
)

service_urls = dict(
    wstesimbrefiscal_homologation='https://wsaduhomoext.afip.gob.ar/diav2/wgestimbrefiscalelectronico/wgestimbrefiscalelectronico.asmx?WSDL',
    wstesimbrefiscal_production='https://wswadu.afip.gob.ar/DIAV2/wgestimbrefiscalelectronico/wgestimbrefiscalelectronico.asmx?WSDL',
    wsct_homologation='https://fwshomo.afip.gov.ar/wsct/CTService?WSDL',
    wsct_production='https://serviciosjava.afip.gob.ar/wsct/CTService?WSDL',
    wsbfev1_homologation='https://wswhomo.afip.gov.ar/wsbfev1/service.asmx?WSDL',
    wsbfev1_production='https://servicios1.afip.gov.ar/wsbfev1/service.asmx?WSDL',
    wsfev1_homologation='https://wswhomo.afip.gov.ar/wsfev1/service.asmx?wsdl',
    wsfev1_production='https://servicios1.afip.gov.ar/wsfev1/service.asmx?wsdl',
    wsfexv1_homologation='https://wswhomo.afip.gov.ar/wsfexv1/service.asmx?wsdl',
    wsfexv1_production='https://servicios1.afip.gov.ar/wsfexv1/service.asmx?wsdl',
    ws_sr_padron_a5_homologation='https://awshomo.afip.gov.ar/sr-padron/webservices/personaServiceA5?WSDL',
    ws_sr_padron_a5_production='https://aws.afip.gov.ar/sr-padron/webservices/personaServiceA5?WSDL',
)
